@@ -20,7 +20,7 @@ class InstaClient:
     CHROMEDRIVER=1
     LOCAHOST=1
     WEB_SERVER=2
-    def __init__(self, driver_type: int=CHROMEDRIVER, host_type:int=LOCAHOST, driver_path=None, debug=False, error_callback=None):
+    def __init__(self, driver_type: int=CHROMEDRIVER, host_type:int=LOCAHOST, driver_path=None, debug=False, error_callback=None, localhost_headless=False):
         """
         Create an `InstaClient` object to access the instagram website.
 
@@ -31,6 +31,7 @@ class InstaClient:
             driver_path (str): The path where you saved the c`hromedriver.exe` file. This is required if you are running the client locally. Defaults to None
             debug (bool): If set to True, the `error_callback` will be called multiple times for debugging or when an error occures. Defaults to Falses
             error_callback (callback): A callback method to be called when an error occures within the InstaClient. Your custom error_callbak must require only one argument named `driver`: a driver like object (The InstaClient will pass itself to the method as `driver` argument)
+            localhost_headless (bool): If set to True, the localhost chromedriver will be set to --headless, meaning it will run without showing the Chrome window. Defaults to true.
 
         Raises:
             InvaildHostError: Raised if host int does not correspond to any host type
@@ -44,15 +45,17 @@ class InstaClient:
         if host_type == self.LOCAHOST and driver_path is None:
             raise InvalidDriverPathError(driver_path)
         self.driver_path = driver_path
-        if error_callback:
+        self.debug = debug
+        if error_callback or debug:
             if not callable(error_callback):
                 raise InvalidErrorCallbackError()
         self.error_callback = error_callback
+        self.localhost_headless = localhost_headless
         self.logged_in = False
         self.driver = None
         self.username = None
         self.password = None
-        self.debug = debug
+        
         self.__init_driver()
 
 
@@ -210,10 +213,17 @@ class InstaClient:
         else:
             # Detect and dismiss save info Dialog
             self.driver.get(ClientUrls.HOME_URL)
-
-            # Detect 'Turn On Notifications' Box
-            if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.NO_NOTIFICATIONS_BTN))):
+            
+            # Detect 'Save to Home Screen' Dialogue
+            if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.DISMISS_DIALOGUE))):
                 self.__dismiss_dialogue()
+            
+            # Detect 'Turn On Notifications' Box
+            if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.DISMISS_DIALOGUE))):
+                self.__dismiss_dialogue()
+                
+            if self.debug:
+                self.error_callback(self.driver)
         return self.logged_in
 
 
@@ -477,13 +487,19 @@ class InstaClient:
         if not self.driver:
             self.__init_driver(login=True)
         # Navigate to User's dm page
+
         try:
+            if self.debug:
+                self.error_callback(self.driver)
             self.nav_user_dm(user)
             text_area = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.DM_TEXT_AREA)))
             text_area.send_keys(message)
             send_btn = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.SEND_DM_BTN)))
             send_btn.click()
         except Exception as error: 
+            if self.debug:
+                self.error_callback(self.driver)
+            print('INSTACLIENT: An error occured when sending a DM to the user <{}>'.format(user))
             if discard_driver:
                 self.__discard_driver()
             raise error
@@ -561,6 +577,40 @@ class InstaClient:
             self.__discard_driver()
         return followers
 
+
+    """ @insta_method # TODO
+    def scrape_dms(self, discard_driver:bool=False):
+        if not self.driver:
+            self.__init_driver()
+
+        # Navigate to DMs
+        if self.username:
+            self.nav_user_dm(self.username)
+        else:
+            raise NotLoggedInError()
+
+        # Infinite Scroll & Loading
+        for i in range(1,count+1):
+            try:
+                div:WebElement = self.driver.find_element_by_xpath(Paths.FOLLOWER_USER_DIV % i)
+                time.sleep(1)
+                username = div.text.split('\n')[0]
+                if  username not in followers:
+                    followers.append(username)
+                if i%callback_frequency==0:
+                    if callback is None:
+                        print('Got another {} followers...'.format(callback_frequency))
+                    else:
+                        callback(*args, **kwargs)
+                self.driver.execute_script("arguments[0].scrollIntoView();", div)
+                # TODO OPTIMIZE ALGORITHM (scroll by more than one account only)
+            except Exception as error:
+                raise error
+
+        
+        if discard_driver:
+            self.__discard_driver() """
+
                 
     # IG UTILITY METHODS
     @insta_method
@@ -658,7 +708,7 @@ class InstaClient:
         
 
     @insta_method
-    def is_valid_user(self, user, nav_to_user=True, discard_driver:bool=False):
+    def is_valid_user(self, user:str, nav_to_user:bool=True, discard_driver:bool=False):
         print('INSTACLIENT: Checking user vadility')
         if not self.driver:
             self.__init_driver(login=True)
@@ -672,6 +722,13 @@ class InstaClient:
         print('INSTACLIENT: Url: ', self.driver.current_url)
         if self.driver.current_url == ClientUrls.LOGIN_THEN_USER.format(user):
             raise NotLoggedInError()
+        elif self.driver.current_url != ClientUrls.NAV_USER.format(user):
+            self.error_callback(self.driver)
+            time.sleep(1)
+            self.driver.get(ClientUrls.NAV_USER.format(user))
+            time.sleep(1)
+            self.error_callback(self.driver)
+
 
         if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.COOKIES_LINK))):
             self.__dismiss_cookies()
@@ -765,14 +822,8 @@ class InstaClient:
             if attempt < 1:
                 if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.COOKIES_LINK))):
                     self.__dismiss_cookies()
-                if not self.check_status():
-                    if url in self.driver.current_url:
-                        self.driver.get(url)
-                        self.__find_element(expectation, url, wait_time, attempt+1)
-                        time.sleep(1)
-                    # Not Logged In!
-                    raise NotLoggedInError()
-                else:
+                
+                if ClientUrls.LOGIN_URL in self.driver.current_url:
                     if attempt < 1 and url is not None:
                         self.driver.get(url)
                         self.__find_element(expectation, url, wait_time, attempt+1)
@@ -780,7 +831,18 @@ class InstaClient:
                         if self.error_callback:
                             self.error_callback(self.driver)
                         raise NoSuchElementException()
+                elif not self.check_status():
+                    if url in self.driver.current_url:
+                        self.driver.get(url)
+                        self.__find_element(expectation, url, wait_time, attempt+1)
+                        time.sleep(1)
+                    # Not Logged In!
+                    if self.error_callback:
+                            self.error_callback(self.driver)
+                    raise NotLoggedInError()   
             else:
+                if self.error_callback:
+                        self.error_callback(self.driver)
                 raise NoSuchElementException()
 
 
@@ -807,6 +869,7 @@ class InstaClient:
 
 
     def __init_driver(self, login=False):
+        print('INSTACLIENT: Initiating Driver...')
         try:
             if self.driver_type == self.CHROMEDRIVER:
                 if self.host_type == self.WEB_SERVER:
@@ -818,13 +881,15 @@ class InstaClient:
                     chrome_options.add_argument("--headless")
                     chrome_options.add_argument("--disable-dev-shm-usage")
                     chrome_options.add_argument("--no-sandbox")
+                    chrome_options.add_experimental_option("excludeSwitches", ["enable-automation"])
+                    chrome_options.add_experimental_option('useAutomationExtension', False)
                     self.driver = webdriver.Chrome(executable_path=os.environ.get("CHROMEDRIVER_PATH"), chrome_options=chrome_options)
                 elif self.host_type == self.LOCAHOST:
                     # Running locally
                     chrome_options = webdriver.ChromeOptions()
                     chrome_options.add_argument('--user-agent=Mozilla/5.0 (iPhone; CPU iPhone OS 10_3 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) CriOS/56.0.2924.75 Mobile/14E5239e Safari/602.1')
                     chrome_options.add_argument("--window-size=343,915")
-                    #chrome_options.add_argument("--headless")
+                    chrome_options.add_argument("--headless") if self.localhost_headless else None
                     chrome_options.add_argument("--disable-dev-shm-usage")
                     chrome_options.add_argument("--no-sandbox")
                     print('Path: ', self.driver_path)
@@ -833,8 +898,9 @@ class InstaClient:
                     raise InvaildHostError(self.host_type)
             else:
                 raise InvaildDriverError(self.driver_type)
-        except Exception as error:
-            raise error
+        except WebDriverException as error:
+            try: self.driver.quit()
+            except: raise error
 
         if login:
             try:
